searchData={"items":[{"type":"module","title":"Guesswork","doc":"Guesswork allows the creation of logical statements and knowledge databases\nwhich then can be used together to figure out answers to questions stated\nin those logical systems.","ref":"Guesswork.html"},{"type":"function","title":"Guesswork.query/3","doc":"Takes a logical statement, a knowledge base, and a target number of answer sets,\nand then returns all entity bindings for which the statement is true, according\nto the knowledge base.\n\nAvailable Options:\n\n* `:label` - Traverses plural entities looking for a specific answer sets.\n  If a list of strings is provided only the supplied variables are traversed. The default value is `false`.\n\n* `:knowledge_base` (`t:term/0`) - The `Guesswork.KnowledgeBase` used to resolve queries. The default value is `Guesswork.EmptyCollection`.\n\n* `:query_id` (`t:String.t/0`) - The id used to report telemetry. Defaults to a new `UUID` (v4).\n\n* `:precompute_count` (`t:integer/0`) - The number of answer sets to compute when resolving a fact.\n  This allows repeated facts (that would result in the same\n  matches) to reuse computation.\n  Note that, with recusive queries, this can lead to infinate searches. The default value is `0`.","ref":"Guesswork.html#query/3"},{"type":"module","title":"Guesswork.Answer","doc":"Answers are maps of variable id to bindings as well as any tests that must be\nchecked.","ref":"Guesswork.Answer.html"},{"type":"type","title":"Guesswork.Answer.bindings/0","doc":"","ref":"Guesswork.Answer.html#t:bindings/0"},{"type":"function","title":"Guesswork.Answer.filter_bindings/2","doc":"Takes an answer set and removes any binding not passed to the function.\n\n`:halt`s are passed through to facilitate stop conditions.","ref":"Guesswork.Answer.html#filter_bindings/2"},{"type":"function","title":"Guesswork.Answer.filter_constraints/2","doc":"Takes an answer set and removes any constraint with inputs not passed to the function.\n\n`:halt`s are passed through to facilitate stop conditions.","ref":"Guesswork.Answer.html#filter_constraints/2"},{"type":"function","title":"Guesswork.Answer.flatten/1","doc":"Extracts the displayable information from the answer set, returning a simple map.","ref":"Guesswork.Answer.html#flatten/1"},{"type":"function","title":"Guesswork.Answer.get_env/1","doc":"Pulls all bound, concrete values from the answer set.","ref":"Guesswork.Answer.html#get_env/1"},{"type":"function","title":"Guesswork.Answer.new/1","doc":"","ref":"Guesswork.Answer.html#new/1"},{"type":"function","title":"Guesswork.Answer.put_binding/4","doc":"","ref":"Guesswork.Answer.html#put_binding/4"},{"type":"function","title":"Guesswork.Answer.put_constraint/3","doc":"","ref":"Guesswork.Answer.html#put_constraint/3"},{"type":"function","title":"Guesswork.Answer.remove_internal_variables/1","doc":"Removes all internal variables that are not currently in use by the constraint store.","ref":"Guesswork.Answer.html#remove_internal_variables/1"},{"type":"function","title":"Guesswork.Answer.remove_resolvable_constraints/1","doc":"Removes all constraints for which the inputs are in the bindings.\nNo constraints are resolved, only removed.","ref":"Guesswork.Answer.html#remove_resolvable_constraints/1"},{"type":"function","title":"Guesswork.Answer.resolve/1","doc":"Resolves both tests and computations as well as checking if the stop conditions\nhave been met and the answer stream should halt.\n\n`:halt`s are passed through to facilitate stop conditions.","ref":"Guesswork.Answer.html#resolve/1"},{"type":"function","title":"Guesswork.Answer.resolve_constraints/2","doc":"","ref":"Guesswork.Answer.html#resolve_constraints/2"},{"type":"function","title":"Guesswork.Answer.substitue_variables/2","doc":"Replaces all bound variables using the variable id to variable map.\n\n`:halt`s are passed through to facilitate stop conditions.","ref":"Guesswork.Answer.html#substitue_variables/2"},{"type":"type","title":"Guesswork.Answer.t/0","doc":"","ref":"Guesswork.Answer.html#t:t/0"},{"type":"function","title":"Guesswork.Answer.traverse_entities/1","doc":"Takes an answer and traverses all entities, looking for specific solutions.\n\nNote that this is very similar to\n[labeling](https://www.swi-prolog.org/pldoc/doc_for?object=labeling/2).","ref":"Guesswork.Answer.html#traverse_entities/1"},{"type":"function","title":"Guesswork.Answer.traverse_entities/2","doc":"Takes an answer and traverses the entities stored in the supplied variables,\nlooking for specific solutions.\n\nNote that this is very similar to\n[labeling](https://www.swi-prolog.org/pldoc/doc_for?object=labeling/2).","ref":"Guesswork.Answer.html#traverse_entities/2"},{"type":"function","title":"Guesswork.Answer.union/2","doc":"Takes two answer sets and attempts to take the union of them.\nIf any variables point to contradicting values the process will fail and\nreturn nil.\n\n`:halt`s are passed through to facilitate stop conditions.","ref":"Guesswork.Answer.html#union/2"},{"type":"module","title":"Guesswork.Answer.Binding","doc":"A 'bound' entity, that is a possible values (notted or exact) and computations,\nconnected to a variable.","ref":"Guesswork.Answer.Binding.html"},{"type":"function","title":"Guesswork.Answer.Binding.get_result_value/1","doc":"","ref":"Guesswork.Answer.Binding.html#get_result_value/1"},{"type":"function","title":"Guesswork.Answer.Binding.merge/2","doc":"Takes two bindings and attempts to merge them.\nAny disagreement between the two will result in failure.","ref":"Guesswork.Answer.Binding.html#merge/2"},{"type":"function","title":"Examples - Guesswork.Answer.Binding.merge/2","doc":"iex> four = Guesswork.Answer.Binding.new(4, false)\n    ...> Guesswork.Answer.Binding.merge(four, four)\n    four\n    iex> five = Guesswork.Answer.Binding.new(5, false)\n    ...> Guesswork.Answer.Binding.merge(four, five)\n    nil\n    iex> not_five = Guesswork.Answer.Binding.new(5, true)\n    ...> Guesswork.Answer.Binding.merge(four, not_five)\n    four\n    iex> not_four = Guesswork.Answer.Binding.new(4, true)\n    ...> Guesswork.Answer.Binding.merge(not_four, not_five)\n    %Guesswork.Answer.Binding{value: {:not, [4, 5]}}","ref":"Guesswork.Answer.Binding.html#merge/2-examples"},{"type":"function","title":"Guesswork.Answer.Binding.new/2","doc":"","ref":"Guesswork.Answer.Binding.html#new/2"},{"type":"type","title":"Guesswork.Answer.Binding.t/0","doc":"","ref":"Guesswork.Answer.Binding.html#t:t/0"},{"type":"type","title":"Guesswork.Answer.Binding.value/0","doc":"","ref":"Guesswork.Answer.Binding.html#t:value/0"},{"type":"module","title":"Guesswork.Answer.Computation","doc":"Represents some arbitrary computation that can be bound to a variable or used to\nverify correctness.","ref":"Guesswork.Answer.Computation.html"},{"type":"function","title":"Guesswork.Answer.Computation.attempt_apply/1","doc":"Takes computation and attempt to apply the inputs to the value.\nSucceeds if all inputs are concrete.","ref":"Guesswork.Answer.Computation.html#attempt_apply/1"},{"type":"type","title":"Guesswork.Answer.Computation.error/0","doc":"","ref":"Guesswork.Answer.Computation.html#t:error/0"},{"type":"type","title":"Guesswork.Answer.Computation.func/0","doc":"","ref":"Guesswork.Answer.Computation.html#t:func/0"},{"type":"function","title":"Guesswork.Answer.Computation.get_vars/1","doc":"","ref":"Guesswork.Answer.Computation.html#get_vars/1"},{"type":"function","title":"Guesswork.Answer.Computation.new/2","doc":"","ref":"Guesswork.Answer.Computation.html#new/2"},{"type":"function","title":"Guesswork.Answer.Computation.run/2","doc":"Checks the environment for the inputs, and then runs the stored functions.\nIf the inputs are not present or not concrete values, an error is returned.","ref":"Guesswork.Answer.Computation.html#run/2"},{"type":"function","title":"Examples - Guesswork.Answer.Computation.run/2","doc":"iex> Guesswork.Answer.Computation.new([], fn -> 4 end)\n    ...> |> Guesswork.Answer.Computation.run(%{})\n    {:ok, 4}\n    iex> Guesswork.Answer.Computation.new([1, 2, 3], fn a, b, c -> a + b + c end)\n    ...> |> Guesswork.Answer.Computation.run(%{})\n    {:ok, 6}\n    iex> var1 = Guesswork.Ast.Variable.new(\"x\")\n    ...> var2 = Guesswork.Ast.Variable.new(\"y\")\n    ...> var3 = Guesswork.Ast.Variable.new(\"z\")\n    ...> \n    ...> Guesswork.Answer.Computation.new([var1, var2, var3], fn a, b, c -> a + b + c end)\n    ...> |> Guesswork.Answer.Computation.run(%{var1 => 1, var2 => 2, var3 => 3})\n    {:ok, 6}\n    iex> Guesswork.Answer.Computation.new([var1, var2, var3], fn a, b, c -> a + b + c end)\n    ...> |> Guesswork.Answer.Computation.run(%{var1 => 1, var3 => 3})\n    {:error, {:unbound, var2}}\n    iex> var4 = Guesswork.Ast.Variable.new(\"a\")\n    ...> \n    ...> Guesswork.Answer.Computation.new([var1, var2, var3], fn a, b, c -> a + b + c end)\n    ...> |> Guesswork.Answer.Computation.run(%{var1 => 1, var2 => var4, var3 => 3})\n    {:error, {:bound_to_var, var2}}","ref":"Guesswork.Answer.Computation.html#run/2-examples"},{"type":"type","title":"Guesswork.Answer.Computation.t/0","doc":"","ref":"Guesswork.Answer.Computation.html#t:t/0"},{"type":"module","title":"Guesswork.Answer.PartiallyEvaluated","doc":"Represents a partially evaluated enumerable of answers, allowing the answer sets\nto be handled one at a time.","ref":"Guesswork.Answer.PartiallyEvaluated.html"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.count_unions/1","doc":"","ref":"Guesswork.Answer.PartiallyEvaluated.html#count_unions/1"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.done?/1","doc":"Checks if the stream has been fully consumed","ref":"Guesswork.Answer.PartiallyEvaluated.html#done?/1"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.evaluated_count/1","doc":"","ref":"Guesswork.Answer.PartiallyEvaluated.html#evaluated_count/1"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.head/1","doc":"If possible, takes the first item in the stream and evaluates it.\nOtherwise marks the value as done.","ref":"Guesswork.Answer.PartiallyEvaluated.html#head/1"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.new/1","doc":"","ref":"Guesswork.Answer.PartiallyEvaluated.html#new/1"},{"type":"function","title":"Guesswork.Answer.PartiallyEvaluated.next_unions/1","doc":"Takes a list of partial evaluations and attempts to calculate the next set of\nunioned answer sets, based on the next evaluated answer set unioned with the\npreviously evaluated answer sets.\n\nThe returned unions can either result in an answer set, or a `:halt`, which\nindicates that a stop condition has been met and that stream should stop.","ref":"Guesswork.Answer.PartiallyEvaluated.html#next_unions/1"},{"type":"type","title":"Guesswork.Answer.PartiallyEvaluated.t/0","doc":"","ref":"Guesswork.Answer.PartiallyEvaluated.html#t:t/0"},{"type":"type","title":"Guesswork.Answer.PartiallyEvaluated.unions_result/0","doc":"","ref":"Guesswork.Answer.PartiallyEvaluated.html#t:unions_result/0"},{"type":"module","title":"Guesswork.Answer.Result","doc":"A module for evaluating and displaying query results.","ref":"Guesswork.Answer.Result.html"},{"type":"type","title":"Guesswork.Answer.Result.t/0","doc":"","ref":"Guesswork.Answer.Result.html#t:t/0"},{"type":"type","title":"Guesswork.Answer.Result.value/0","doc":"","ref":"Guesswork.Answer.Result.html#t:value/0"},{"type":"function","title":"Guesswork.Answer.Result.value_to_string/1","doc":"Takes a value and produces a readable string.","ref":"Guesswork.Answer.Result.html#value_to_string/1"},{"type":"module","title":"Guesswork.Ast","doc":"Helper tools/macros for building rules and statements.","ref":"Guesswork.Ast.html"},{"type":"macro","title":"Guesswork.Ast.fact/2","doc":"Creates a `Guesswork.Ast.Fact` that allows the use of variables.","ref":"Guesswork.Ast.html#fact/2"},{"type":"macro","title":"Guesswork.Ast.is/2","doc":"Builds a `Guesswork.Ast.Is` using a `fn`.\n\nNote that this cannot be used in a `Guessork.KnowledgeBase.Collection`.\nInstead `Guesswork.Ast.Is.new/3` will have to be used directly with a function\ncaputured using `&Mod.fun/arity`.","ref":"Guesswork.Ast.html#is/2"},{"type":"macro","title":"Guesswork.Ast.rule/3","doc":"Creates a `Guesswork.Ast.Rule` that, like `term`, allows the use of variables.","ref":"Guesswork.Ast.html#rule/3"},{"type":"macro","title":"Guesswork.Ast.term/1","doc":"Allows a `Guesswork.Ast.Term` to be built with variables.\n\nNote that, like ecto queries, external values and elixir expressions can be\ninjected by using the pin operator (`^`).","ref":"Guesswork.Ast.html#term/1"},{"type":"macro","title":"Examples - Guesswork.Ast.term/1","doc":"iex> Guesswork.Ast.term(b)\n    Guesswork.Ast.Variable.new(\"b\")\n    iex> Guesswork.Ast.term(Guesswork.Ast.Fact.new(\"test\", [a, :b]))\n    Guesswork.Ast.Fact.new(\"test\", [Guesswork.Ast.Variable.new(\"a\"), :b])\n    iex> a = 45\n    ...> Guesswork.Ast.term(Guesswork.Ast.Fact.new(\"test\", [^a, :b]))\n    Guesswork.Ast.Fact.new(\"test\", [45, :b])","ref":"Guesswork.Ast.html#term/1-examples"},{"type":"module","title":"Guesswork.Ast.And","doc":"Simple conjunction.","ref":"Guesswork.Ast.And.html"},{"type":"function","title":"Guesswork.Ast.And.new/1","doc":"","ref":"Guesswork.Ast.And.html#new/1"},{"type":"type","title":"Guesswork.Ast.And.t/0","doc":"","ref":"Guesswork.Ast.And.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Assign","doc":"A simple assignment of a value to a variable.","ref":"Guesswork.Ast.Assign.html"},{"type":"function","title":"Guesswork.Ast.Assign.new/2","doc":"","ref":"Guesswork.Ast.Assign.html#new/2"},{"type":"type","title":"Guesswork.Ast.Assign.t/0","doc":"","ref":"Guesswork.Ast.Assign.html#t:t/0"},{"type":"protocol","title":"Guesswork.Ast.Entity","doc":"An entity is a concrete value or set of possible values (plural type) that can\nbe bound to a variable or included as an argument to a `Guesswork.Ast.Fact`.","ref":"Guesswork.Ast.Entity.html"},{"type":"function","title":"Guesswork.Ast.Entity.compare/2","doc":"Compares two entities.\n\nNote that there may be an implicit assignment (followed by merge) durring an\ncommparision.\nThis is why instead of `:eq` an entity canidate is returned.\nThis canidate may be 'smaller' than the inputs, allowing a better value to be\nbound.\n\nPlural values are also why `:neq` may be returned.\nFor instance, consider the sets `[1, 4, 5]` and `[2, 3, 6]`.\nThere is clearly no instance where these sets are equal, but you can not say\none will always be less or greater than the other.","ref":"Guesswork.Ast.Entity.html#compare/2"},{"type":"function","title":"Guesswork.Ast.Entity.display/1","doc":"Display the entity for the result set.","ref":"Guesswork.Ast.Entity.html#display/1"},{"type":"function","title":"Guesswork.Ast.Entity.merge/2","doc":"Merge two entities have the same negation.\nNote that they do not need to have the same type.\nFor instance, a plural type can be compared to a single type (in that argument\norder) and should return the single type if it is a member of the plural type.\nThis generally requires using the implementation of the plural type.","ref":"Guesswork.Ast.Entity.html#merge/2"},{"type":"function","title":"Guesswork.Ast.Entity.plural?/1","doc":"Indicates if the entity represents multiple possible values or a single definate\nvalue.\nNote that the `merge/2` implementation of plural entities will always be prefered.","ref":"Guesswork.Ast.Entity.html#plural?/1"},{"type":"type","title":"Guesswork.Ast.Entity.t/0","doc":"All the types that implement this protocol.","ref":"Guesswork.Ast.Entity.html#t:t/0"},{"type":"function","title":"Guesswork.Ast.Entity.traverse/1","doc":"Allows a user to traverse an entity.\nMostly used for plural values that are `Enumerable`.\nConcrete values should just wrap themselves in a list.","ref":"Guesswork.Ast.Entity.html#traverse/1"},{"type":"module","title":"Guesswork.Ast.Fact","doc":"A fact is just a relationship (string) and some entities (concrete or non-concrete)\nand states that the relationship hold for those values in that order.","ref":"Guesswork.Ast.Fact.html"},{"type":"type","title":"Guesswork.Ast.Fact.arg/0","doc":"","ref":"Guesswork.Ast.Fact.html#t:arg/0"},{"type":"function","title":"Guesswork.Ast.Fact.get_metadata/1","doc":"Extracts the relationship name and arity.","ref":"Guesswork.Ast.Fact.html#get_metadata/1"},{"type":"function","title":"Guesswork.Ast.Fact.match/3","doc":"Takes two facts and attempt to 'match' them by ensuring that their relationships\nmatch, all concrete terms match, and then binding any variables in the first fact\nto values in the opposite entity.\nReturns nil if the match fails, an answer set if it succeeds.\n\nNote that the same concrete fact matched on itself will result in an empty answer\nset and that variables match variables.","ref":"Guesswork.Ast.Fact.html#match/3"},{"type":"function","title":"Examples - Guesswork.Ast.Fact.match/3","doc":"iex> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> Guesswork.Ast.Fact.match(f1, f2, \"test\")\n    Guesswork.Answer.new(\"test\")\n    iex> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :d])\n    ...> Guesswork.Ast.Fact.match(f1, f2, \"test\")\n    nil\n    iex> var1 = Guesswork.Ast.Variable.new(\"x\")\n    ...> var2 = Guesswork.Ast.Variable.new(\"y\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [var1, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, var2])\n    ...> Guesswork.Ast.Fact.match(f1, f2, \"test\")\n    %Guesswork.Answer{\n      query_id: \"test\",\n      bindings: %{\n        \"x\" => Guesswork.Answer.Binding.new(:a, false)\n      }\n    }\n    iex> var = Guesswork.Ast.Variable.new(\"x\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [var, :b, var])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> Guesswork.Ast.Fact.match(f1, f2, \"test\")\n    nil\n    iex> var = Guesswork.Ast.Variable.new(\"x\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, var])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, var, var])\n    ...> Guesswork.Ast.Fact.match(f1, f2, \"test\")\n    %Guesswork.Answer{\n      query_id: \"test\",\n      bindings: %{\n        \"x\" => Guesswork.Answer.Binding.new(:b, false)\n      }\n    }","ref":"Guesswork.Ast.Fact.html#match/3-examples"},{"type":"type","title":"Guesswork.Ast.Fact.metadata/0","doc":"","ref":"Guesswork.Ast.Fact.html#t:metadata/0"},{"type":"function","title":"Guesswork.Ast.Fact.new/2","doc":"","ref":"Guesswork.Ast.Fact.html#new/2"},{"type":"function","title":"Guesswork.Ast.Fact.substitute/2","doc":"Attempts to substitute the values in the second fact for any variables in the\nfirst.\nReturns nil if the relation doesn't match, any concrete entities don't match,\nor if the second fact attempts to substitute a variable for a concrete value.","ref":"Guesswork.Ast.Fact.html#substitute/2"},{"type":"function","title":"Example - Guesswork.Ast.Fact.substitute/2","doc":"iex> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:other, [:a, :b, :c])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    nil\n    iex> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    %{}\n    iex> var = Guesswork.Ast.Variable.new(\"x\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, var, :c])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    nil\n    iex> var1 = Guesswork.Ast.Variable.new(\"x\")\n    ...> var2 = Guesswork.Ast.Variable.new(\"y\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [:a, var1, var2])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :c])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    %{var1 => :b, var2 => :c}\n    iex> var1 = Guesswork.Ast.Variable.new(\"x\")\n    ...> var2 = Guesswork.Ast.Variable.new(\"y\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [:a, var1, :c])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, var2, :c])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    %{var1 => var2}\n    iex> var1 = Guesswork.Ast.Variable.new(\"x\")\n    ...> var2 = Guesswork.Ast.Variable.new(\"y\")\n    ...> f1 = Guesswork.Ast.Fact.new(:thing, [:a, var1, var2])\n    ...> f2 = Guesswork.Ast.Fact.new(:thing, [:a, :b, :b])\n    ...> Guesswork.Ast.Fact.substitute(f1, f2)\n    %{var1 => :b, var2 => :b}","ref":"Guesswork.Ast.Fact.html#substitute/2-example"},{"type":"type","title":"Guesswork.Ast.Fact.t/0","doc":"","ref":"Guesswork.Ast.Fact.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Fact.Precompute","doc":"Used by `Guesswork.Ast.Fact` to cache streams where a portion of those streams\nare calculated in advance.","ref":"Guesswork.Ast.Fact.Precompute.html"},{"type":"function","title":"Guesswork.Ast.Fact.Precompute.build_stream/5","doc":"","ref":"Guesswork.Ast.Fact.Precompute.html#build_stream/5"},{"type":"function","title":"Guesswork.Ast.Fact.Precompute.get_stream/3","doc":"","ref":"Guesswork.Ast.Fact.Precompute.html#get_stream/3"},{"type":"function","title":"Guesswork.Ast.Fact.Precompute.new_cache/0","doc":"","ref":"Guesswork.Ast.Fact.Precompute.html#new_cache/0"},{"type":"module","title":"Guesswork.Ast.Fail","doc":"A statement that always produces an empty answer set.","ref":"Guesswork.Ast.Fail.html"},{"type":"function","title":"Guesswork.Ast.Fail.new/0","doc":"","ref":"Guesswork.Ast.Fail.html#new/0"},{"type":"type","title":"Guesswork.Ast.Fail.t/0","doc":"","ref":"Guesswork.Ast.Fail.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Is","doc":"Binds a variable or entity to a function (constraint).\nIf a variable is used, the constraint is used to figure out its value.\nIf an entity is used, the constraint is used to test validity.\n\nNote that, if all the `inputs` are concrete, the computation for is can be\nresolved before `Guesswork.Answer.Result.run/2` is called.","ref":"Guesswork.Ast.Is.html"},{"type":"function","title":"Guesswork.Ast.Is.new/4","doc":"Builds a new constraint.\n\nThe following fields are available:\n\n* `:halt_on_error` (`t:term/0`) - Indicates that, should this constraint produce a value that\n  disagrees with the answer set or is invalid, `Guesswork` should\n  stop searching. The default value is `false`.","ref":"Guesswork.Ast.Is.html#new/4"},{"type":"type","title":"Guesswork.Ast.Is.opts_list/0","doc":"","ref":"Guesswork.Ast.Is.html#t:opts_list/0"},{"type":"type","title":"Guesswork.Ast.Is.t/0","doc":"","ref":"Guesswork.Ast.Is.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Not","doc":"Simple negation.","ref":"Guesswork.Ast.Not.html"},{"type":"function","title":"Guesswork.Ast.Not.new/1","doc":"","ref":"Guesswork.Ast.Not.html#new/1"},{"type":"type","title":"Guesswork.Ast.Not.t/0","doc":"","ref":"Guesswork.Ast.Not.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.OneOf","doc":"Binds a value to an `Enumerable` of possible values.\n\nNote that, when created, the bound value should always be a variable.\nHowever, in the process of substitution it may become a concrete value, in\nwhich case, on resolution, the stored value is checked against the stream.\n\nIt is invalid for the stream to produce a variable.","ref":"Guesswork.Ast.OneOf.html"},{"type":"function","title":"Guesswork.Ast.OneOf.new/2","doc":"","ref":"Guesswork.Ast.OneOf.html#new/2"},{"type":"type","title":"Guesswork.Ast.OneOf.t/0","doc":"","ref":"Guesswork.Ast.OneOf.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Or","doc":"Simple disjunction.","ref":"Guesswork.Ast.Or.html"},{"type":"function","title":"Guesswork.Ast.Or.new/1","doc":"","ref":"Guesswork.Ast.Or.html#new/1"},{"type":"type","title":"Guesswork.Ast.Or.t/0","doc":"","ref":"Guesswork.Ast.Or.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Rule","doc":"A rule maps a (ideally non-concrete) fact to a defining statement.","ref":"Guesswork.Ast.Rule.html"},{"type":"function","title":"Guesswork.Ast.Rule.get_metadata/1","doc":"","ref":"Guesswork.Ast.Rule.html#get_metadata/1"},{"type":"function","title":"Guesswork.Ast.Rule.new/2","doc":"","ref":"Guesswork.Ast.Rule.html#new/2"},{"type":"function","title":"Guesswork.Ast.Rule.simplify_definition/1","doc":"","ref":"Guesswork.Ast.Rule.html#simplify_definition/1"},{"type":"function","title":"Guesswork.Ast.Rule.substitute/2","doc":"Attempts to substitute the values in the fact for any variables in the rule's\nheader.\nReturns nil if the relation doesn't match, any concrete entities don't match,\nor if attempting to substitute a variable for a concrete value.","ref":"Guesswork.Ast.Rule.html#substitute/2"},{"type":"type","title":"Guesswork.Ast.Rule.t/0","doc":"","ref":"Guesswork.Ast.Rule.html#t:t/0"},{"type":"protocol","title":"Guesswork.Ast.Statement","doc":"A Statement is a semantically complete value.\nIt is in no way required to be logically consistent.","ref":"Guesswork.Ast.Statement.html"},{"type":"type","title":"Guesswork.Ast.Statement.call/0","doc":"","ref":"Guesswork.Ast.Statement.html#t:call/0"},{"type":"type","title":"Guesswork.Ast.Statement.env/0","doc":"","ref":"Guesswork.Ast.Statement.html#t:env/0"},{"type":"function","title":"Guesswork.Ast.Statement.find_statements/2","doc":"Searches all clauses of the statement looking for values that match the predicate.","ref":"Guesswork.Ast.Statement.html#find_statements/2"},{"type":"function","title":"Guesswork.Ast.Statement.resolve/2","doc":"Takes a statement (ideally non-concrete) and a knowledge base and attempts to\nfind any permutations of bindings for the variables that are true according to\nthe knowledge base and statement.\n\nEither returns a stream of possible answer sets, or returns a closure that takes\nan environment and returns a stream of possible answer sets.","ref":"Guesswork.Ast.Statement.html#resolve/2"},{"type":"function","title":"Guesswork.Ast.Statement.simplify/1","doc":"Attempts to simplify the statement into a less nested operation and returns nil\nif the statement holds no logical information.\n\nNote that the only requirement for this function is that the input and output\nstatements are logically equivalent, not that the result is the shallowest\npossible answer (in practice its quite 'greedy' so it is unlikely to produce\nthe 'best' answer, just better).","ref":"Guesswork.Ast.Statement.html#simplify/1"},{"type":"type","title":"Guesswork.Ast.Statement.stream/0","doc":"","ref":"Guesswork.Ast.Statement.html#t:stream/0"},{"type":"function","title":"Guesswork.Ast.Statement.substitute/2","doc":"Takes an environment (a mapping of variables to entities) and substitutes the\nentities into any place the variable is.","ref":"Guesswork.Ast.Statement.html#substitute/2"},{"type":"type","title":"Guesswork.Ast.Statement.t/0","doc":"All the types that implement this protocol.","ref":"Guesswork.Ast.Statement.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Statement.Opts","doc":"Defines the options used by the `Guesswork.Ast.Statement` protocol, as well as\nthe current state of the query.\n\nThe following fields are available:\n\n* `:knowledge_base` (`t:term/0`) - The `Guesswork.KnowledgeBase` used to resolve queries. The default value is `Guesswork.EmptyCollection`.\n\n* `:query_id` (`t:String.t/0`) - The id used to report telemetry. Defaults to a new `UUID` (v4).\n\n* `:precompute_count` (`t:integer/0`) - The number of answer sets to compute when resolving a fact.\n  This allows repeated facts (that would result in the same\n  matches) to reuse computation.\n  Note that, with recusive queries, this can lead to infinate searches. The default value is `0`.\n\n\n\n## `negated` Vs. `negate_constraints`\n\n`Elixir.Guesswork.Ast.Statement.Opts` tracks two different negated values: `negated` and `negate_constraints`.\n`negated` is used for resolving facts either to facts and falsehoods and is only changed\nwhen a `Guesswork.Ast.Not` is used.\n`negate_constraints` is used to figure out what the result of a constraint should be and\nis changed by `Guesswork.Ast.Not` and then reset when rules are resolved.","ref":"Guesswork.Ast.Statement.Opts.html"},{"type":"function","title":"Guesswork.Ast.Statement.Opts.inc_call_depth/1","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#inc_call_depth/1"},{"type":"function","title":"Guesswork.Ast.Statement.Opts.negate/1","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#negate/1"},{"type":"function","title":"Guesswork.Ast.Statement.Opts.new/1","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#new/1"},{"type":"type","title":"Guesswork.Ast.Statement.Opts.opts_list/0","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#t:opts_list/0"},{"type":"function","title":"Guesswork.Ast.Statement.Opts.reset_negate_constraints/1","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#reset_negate_constraints/1"},{"type":"function","title":"Guesswork.Ast.Statement.Opts.schema_list/0","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#schema_list/0"},{"type":"type","title":"Guesswork.Ast.Statement.Opts.t/0","doc":"","ref":"Guesswork.Ast.Statement.Opts.html#t:t/0"},{"type":"protocol","title":"Guesswork.Ast.Term","doc":"A Term is any possible, computable value.\nNote that a term doesn't necessarily made any sense or have any semantic value.","ref":"Guesswork.Ast.Term.html"},{"type":"function","title":"Guesswork.Ast.Term.concrete?/1","doc":"Check if a term is concrete, that is that is has no variables in it.","ref":"Guesswork.Ast.Term.html#concrete?/1"},{"type":"function","title":"Guesswork.Ast.Term.get_variables/1","doc":"Pulls all variables from the term.","ref":"Guesswork.Ast.Term.html#get_variables/1"},{"type":"type","title":"Guesswork.Ast.Term.t/0","doc":"All the types that implement this protocol.","ref":"Guesswork.Ast.Term.html#t:t/0"},{"type":"module","title":"Guesswork.Ast.Variable","doc":"Indicates an unbound entity (also called a non-concrete value).","ref":"Guesswork.Ast.Variable.html"},{"type":"function","title":"Guesswork.Ast.Variable.new/1","doc":"","ref":"Guesswork.Ast.Variable.html#new/1"},{"type":"type","title":"Guesswork.Ast.Variable.t/0","doc":"","ref":"Guesswork.Ast.Variable.html#t:t/0"},{"type":"protocol","title":"Guesswork.Constraint","doc":"Represents a relationship between values that can be evaluated to produce or test\nnew values.","ref":"Guesswork.Constraint.html"},{"type":"function","title":"Guesswork.Constraint.apply/2","doc":"Applies the known values to the constraint, attempting to build a new value.","ref":"Guesswork.Constraint.html#apply/2"},{"type":"function","title":"Guesswork.Constraint.describe/1","doc":"Displays the constraint in a human readable way.\nOpaque values should return `nil`.","ref":"Guesswork.Constraint.html#describe/1"},{"type":"function","title":"Guesswork.Constraint.inputs/1","doc":"Pulls all possible sets of inputs that could resolve the constraint.","ref":"Guesswork.Constraint.html#inputs/1"},{"type":"type","title":"Guesswork.Constraint.result/0","doc":"A simple value that indicates the result of a constraint.\n\n- `:error` if the constraint is invalid.\n- `:noanswer` if the constraint cannot be resolved.\n- `:ok` if the constraint resolves to true.\n- `%Guesswork.Constraint.Result{}` if a new value has been derived.\n  The last term indicates if the term should be negated.","ref":"Guesswork.Constraint.html#t:result/0"},{"type":"type","title":"Guesswork.Constraint.t/0","doc":"All the types that implement this protocol.","ref":"Guesswork.Constraint.html#t:t/0"},{"type":"module","title":"Guesswork.Constraint.Arithmetic","doc":"A constraint that represents basic integer arithmetic.","ref":"Guesswork.Constraint.Arithmetic.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.add/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#add/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.build_fn/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#build_fn/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.divide/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#divide/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.multiply/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#multiply/2"},{"type":"macro","title":"Guesswork.Constraint.Arithmetic.new/2","doc":"Builds a new arithmetic constraint from an elixir expression.\n\nThe following fields are available:\n\n* `:halt_on_error` (`t:term/0`) - Indicates that, should this constraint produce a value that\n  disagrees with the answer set or is invalid, `Guesswork` should\n  stop searching. The default value is `false`.","ref":"Guesswork.Constraint.Arithmetic.html#new/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.new/6","doc":"Takes an elixir polynomial expression that contains exactly one `=`, simplifies,\nand returns a logical statement.","ref":"Guesswork.Constraint.Arithmetic.html#new/6"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.operation/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#t:operation/0"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.opts_list/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#t:opts_list/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.subtract/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#subtract/2"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.t/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#t:t/0"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.value/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.html#t:value/0"},{"type":"module","title":"Guesswork.Constraint.Arithmetic.InternalVariable","doc":"This module provides a simple tool for building unique, readable, internal\nvariables (used when polynomials are split up in to smaller sub statements).\nIf the Agent is running then a simple counter (turned into a base36 string)\nis used, guaranteeing that the shortest possible unique variable.\nHowever, if the Agent is not running then a random string is defaulted to.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.InternalVariable.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.html#child_spec/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.InternalVariable.get/0","doc":"Attempts to get a variable sutiable for internal use.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.html#get/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.InternalVariable.reset/0","doc":"Resets the counter, starts at `_A`.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.html#reset/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.InternalVariable.start_link/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.html#start_link/1"},{"type":"module","title":"Guesswork.Constraint.Arithmetic.InternalVariable.Base36","doc":"A simple module for building the base36 string version of an integer.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.Base36.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate/1","doc":"Returns the base 36 string of the the integer prefixed with `_`.","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.Base36.html#calculate/1"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate/1","doc":"iex> Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate(0)\n  \"_A\"\n  iex> Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate(15)\n  \"_P\"\n  iex> Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate(27)\n  \"_2\"\n  iex> Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate(51)\n  \"_AP\"\n  iex> Guesswork.Constraint.Arithmetic.InternalVariable.Base36.calculate(1118)\n  \"_5C\"","ref":"Guesswork.Constraint.Arithmetic.InternalVariable.Base36.html#calculate/1-examples"},{"type":"module","title":"Guesswork.Constraint.Arithmetic.Polynomial","doc":"A canonical representation of a polynomial.\nImplicitly equal to zero\n\nTaken from Peter Norvig's Paradigm of Artificial Intelligence Programming: Case\nStudies in Common Lisp, Chapter 15.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.build_logical_statement/3","doc":"Translates the polynomial into logical statements.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#build_logical_statement/3"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.decompose/3","doc":"breaks a polynomial in 2 or 3 term statements.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#decompose/3"},{"type":"macro","title":"Guesswork.Constraint.Arithmetic.Polynomial.equation/1","doc":"Builds a canonical polynomial from an elixir expression.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#equation/1"},{"type":"macro","title":"Examples - Guesswork.Constraint.Arithmetic.Polynomial.equation/1","doc":"iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(5)\n  {5, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(x)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{1 => 1}\n   }, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(-x)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{1 => -1}\n   }, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(y - x)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{\n       0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n         var: %Guesswork.Ast.Variable{display: \"y\"},\n         coefficients: %{1 => 1}\n       },\n       1 => -1\n     }\n   }, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(y * 3 - x * x + 5)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{\n       0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n         var: %Guesswork.Ast.Variable{display: \"y\"},\n         coefficients: %{0 => 5, 1 => 3}\n       },\n       2 => -1\n     }\n   }, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(y * 3 - x * x + 5 = 4)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{\n       0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n         var: %Guesswork.Ast.Variable{display: \"y\"},\n         coefficients: %{0 => 1, 1 => 3}\n       },\n       2 => -1\n     }\n   }, :=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(x < 3)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{0 => -3, 1 => 1}\n   }, :<}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(x <= 3)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{0 => -3, 1 => 1}\n   }, :<=}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(x > 3)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{0 => -3, 1 => 1}\n   }, :>}\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.equation(x >= 3)\n  {%Guesswork.Constraint.Arithmetic.Polynomial{\n     var: %Guesswork.Ast.Variable{display: \"x\"},\n     coefficients: %{0 => -3, 1 => 1}\n   }, :>=}","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#equation/1-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.negate/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#negate/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.new/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#new/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.new/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#new/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.normalize/1","doc":"Removes any `0` coefficents from the polynomial.\nNote that this does not recurse.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#normalize/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.plus/2","doc":"Adds two polynomials.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#plus/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Polynomial.plus/2","doc":"iex> Guesswork.Constraint.Arithmetic.Polynomial.plus(3, 7)\n  10\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(7)\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => 10, 1 => 2, 2 => 1}\n  }\n  iex> -7\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => -4, 1 => 2, 2 => 1}\n  }\n  iex> -3\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{1 => 2, 2 => 1}\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [-5, -2, 0, 3])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => -2, 2 => 1, 3 => 3}\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [-5, 1, 0, 3])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => -2, 1 => 3, 2 => 1, 3 => 3}\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [-5, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{\n      0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{0 => -2, 1 => 1}\n      },\n      1 => 2,\n      2 => 1\n    }\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [-5, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.plus(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [6, -1, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{\n      0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{0 => 4, 2 => 1}\n      },\n      1 => 2,\n      2 => 1\n    }\n  }","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#plus/2-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.pop_coefficient/3","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#pop_coefficient/3"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.pop_constant_term/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#pop_constant_term/1"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.Polynomial.t/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#t:t/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Polynomial.times/2","doc":"Multiplies two polynomial.","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#times/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Polynomial.times/2","doc":"iex> Guesswork.Constraint.Arithmetic.Polynomial.times(3, 7)\n  21\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, -1, 2])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(3)\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => 9, 1 => -3, 2 => 6}\n  }\n  iex> 0\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, -1, 2])\n  ...> )\n  0\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [5, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{0 => 15, 1 => 13, 2 => 7, 3 => 1}\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [5, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{\n      0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{0 => 15, 1 => 3}\n      },\n      1 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{0 => 10, 1 => 2}\n      },\n      2 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{0 => 5, 1 => 1}\n      }\n    }\n  }\n  iex> Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [5, 1])\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"x\", [3, 2, 1])\n  ...> )\n  ...> |> Guesswork.Constraint.Arithmetic.Polynomial.times(\n  ...>   Guesswork.Constraint.Arithmetic.Polynomial.new(\"y\", [0, 2])\n  ...> )\n  %Guesswork.Constraint.Arithmetic.Polynomial{\n    var: %Guesswork.Ast.Variable{display: \"x\"},\n    coefficients: %{\n      0 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{1 => 30, 2 => 6}\n      },\n      1 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{1 => 20, 2 => 4}\n      },\n      2 => %Guesswork.Constraint.Arithmetic.Polynomial{\n        var: %Guesswork.Ast.Variable{display: \"y\"},\n        coefficients: %{1 => 10, 2 => 2}\n      }\n    }\n  }","ref":"Guesswork.Constraint.Arithmetic.Polynomial.html#times/2-examples"},{"type":"module","title":"Guesswork.Constraint.Arithmetic.Range","doc":"Indicates multiple possible integer values.\n\n# Negation\n\nBecause ranges are subsets of the denumerable integer set, it is possible to\nrepresent `Not  ` as the complement of that range.\n\n# Enumerable\n\nBecause ranges can represent infinite sets you should be very careful when using\nthe `Enum` module.\nIn general, `Stream` is going to be a lot safer.","ref":"Guesswork.Constraint.Arithmetic.Range.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.add/2","doc":"Adds all of the points in one range to all of the points of another.","ref":"Guesswork.Constraint.Arithmetic.Range.html#add/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.add/2","doc":"iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.add(r1, r2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{7, 7}]}\n    iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, 7, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.add(r1, r2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{7, 10}]}\n    iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, :pos_inf, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, 7, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.add(r1, r2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{7, :pos_inf}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#add/2-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.all_points?/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#all_points?/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.complement/1","doc":"Takes the complement of the set.\nReturn `nil` for an empty range.","ref":"Guesswork.Constraint.Arithmetic.Range.html#complement/1"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.complement/1","doc":"iex> %Guesswork.Constraint.Arithmetic.Range{values: []}\n    ...> |> Guesswork.Constraint.Arithmetic.Range.complement()\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, :pos_inf}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(:neg_inf, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.complement()\n    nil\n    iex> Guesswork.Constraint.Arithmetic.Range.new(:neg_inf, 4, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(7, :pos_inf)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.complement()\n    %Guesswork.Constraint.Arithmetic.Range{values: [{5, 6}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(-3, 4, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(7, 11)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(35, 98)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.complement()\n    %Guesswork.Constraint.Arithmetic.Range{\n      values: [{:neg_inf, -4}, {5, 6}, {12, 34}, {99, :pos_inf}]\n    }","ref":"Guesswork.Constraint.Arithmetic.Range.html#complement/1-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.count/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#count/1"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.Range.elem/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#t:elem/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.insert/2","doc":"Inserts a single integer point into a range.","ref":"Guesswork.Constraint.Arithmetic.Range.html#insert/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.insert/2","doc":"iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(4)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, 1}, {4, 4}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(1)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{1, :pos_inf}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(-11)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, 1}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#insert/2-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.insert/3","doc":"Inserts set of points into an existing range.","ref":"Guesswork.Constraint.Arithmetic.Range.html#insert/3"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.insert/3","doc":"iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(4, 2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, 4}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(-1, 3)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{-1, :pos_inf}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.insert(-11, :neg_inf)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, 1}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#insert/3-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.intersection/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#intersection/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.max/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#max/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.member?/2","doc":"Checks if the range contains the integer.","ref":"Guesswork.Constraint.Arithmetic.Range.html#member?/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.member?/2","doc":"iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.member?(2)\n    false\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.member?(2)\n    true\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.member?(3)\n    false","ref":"Guesswork.Constraint.Arithmetic.Range.html#member?/2-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.min/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#min/1"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.multiply_int/2","doc":"Multiplies the range by an integer value.","ref":"Guesswork.Constraint.Arithmetic.Range.html#multiply_int/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.new/0","doc":"Builds a new range that covers every possible integer.","ref":"Guesswork.Constraint.Arithmetic.Range.html#new/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.new/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#new/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.new/3","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#new/3"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.Range.point/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#t:point/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.point_greater_than?/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#point_greater_than?/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.point_less_than?/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#point_less_than?/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.remove/2","doc":"Removes a single integer point from a range.","ref":"Guesswork.Constraint.Arithmetic.Range.html#remove/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.remove/2","doc":"iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(1)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, 0}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{3, :pos_inf}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(-4)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, -5}, {-3, 1}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#remove/2-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.remove/3","doc":"Removes a set of points from a range.\nReturns nil if the the set becomes empty.","ref":"Guesswork.Constraint.Arithmetic.Range.html#remove/3"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.remove/3","doc":"iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(:neg_inf, 1)\n    nil\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, false)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(:neg_inf, 4)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{5, :pos_inf}]}\n    iex> Guesswork.Constraint.Arithmetic.Range.new(2, :pos_inf, true)\n    ...> |> Guesswork.Constraint.Arithmetic.Range.remove(-8, -4)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{:neg_inf, -9}, {-3, 1}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#remove/3-examples"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.subtract/2","doc":"Subtracts all of the points in one range to all of the points of another.","ref":"Guesswork.Constraint.Arithmetic.Range.html#subtract/2"},{"type":"function","title":"Examples - Guesswork.Constraint.Arithmetic.Range.subtract/2","doc":"iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.subtract(r1, r2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{-1, -1}]}\n    iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, 7, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.subtract(r2, r1)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{1, 4}]}\n    iex> r1 = Guesswork.Constraint.Arithmetic.Range.new(3, :pos_inf, false)\n    ...> r2 = Guesswork.Constraint.Arithmetic.Range.new(4, 7, false)\n    ...> Guesswork.Constraint.Arithmetic.Range.subtract(r1, r2)\n    %Guesswork.Constraint.Arithmetic.Range{values: [{-4, :pos_inf}]}","ref":"Guesswork.Constraint.Arithmetic.Range.html#subtract/2-examples"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.Range.t/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#t:t/0"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Range.union/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Range.html#union/2"},{"type":"module","title":"Guesswork.Constraint.Arithmetic.Statement","doc":"Stores an aritmmetic statement in `a + b = c`, `a * b = c`, or `a = c`.","ref":"Guesswork.Constraint.Arithmetic.Statement.html"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Statement.new/3","doc":"","ref":"Guesswork.Constraint.Arithmetic.Statement.html#new/3"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Statement.new/5","doc":"","ref":"Guesswork.Constraint.Arithmetic.Statement.html#new/5"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Statement.put_halt_on_error/2","doc":"","ref":"Guesswork.Constraint.Arithmetic.Statement.html#put_halt_on_error/2"},{"type":"function","title":"Guesswork.Constraint.Arithmetic.Statement.simplify/1","doc":"","ref":"Guesswork.Constraint.Arithmetic.Statement.html#simplify/1"},{"type":"type","title":"Guesswork.Constraint.Arithmetic.Statement.t/0","doc":"","ref":"Guesswork.Constraint.Arithmetic.Statement.html#t:t/0"},{"type":"module","title":"Guesswork.Constraint.Comparison","doc":"Simple 2-ary comparisions of terms implemented as a `Guesswork.Constraint`.\n\n# Equality vs. Other Constraints\n\nThe `:=` constraint is different from the other constraints because it implies\na logical relationship strong enough to bind a value.\nThe other constraints will only compare values and result in an assessment of\nan existing bingding's validity.\n\n# Comparing Plural Values\n\nBecause plural values represent multiple possible values, saying they are 'equal'\nis not possible.\nHowever it can be said that they can resolve to an equal value, or that they\nare strictly less/greater than (`:=`, `:<`, and `:>`).\nIn cases other than this many comparisions will result in `:noanswer` out of\nsafety.\n\n# Comparing Integers\n\nWhen comparing a variable to an integer (such as `x < 4`) and using comparision\nas a statement, resolution will result in a `Guesswork.Constraint.Arithmetic.Range`.","ref":"Guesswork.Constraint.Comparison.html"},{"type":"function","title":"Guesswork.Constraint.Comparison.apply/1","doc":"Applies a comparison constraint.","ref":"Guesswork.Constraint.Comparison.html#apply/1"},{"type":"function","title":"Guesswork.Constraint.Comparison.apply/4","doc":"Applies a comparison constraint to the arguments.","ref":"Guesswork.Constraint.Comparison.html#apply/4"},{"type":"type","title":"Guesswork.Constraint.Comparison.constraint/0","doc":"","ref":"Guesswork.Constraint.Comparison.html#t:constraint/0"},{"type":"function","title":"Guesswork.Constraint.Comparison.negate_constraint/2","doc":"","ref":"Guesswork.Constraint.Comparison.html#negate_constraint/2"},{"type":"function","title":"Guesswork.Constraint.Comparison.new/4","doc":"","ref":"Guesswork.Constraint.Comparison.html#new/4"},{"type":"function","title":"Guesswork.Constraint.Comparison.print/2","doc":"","ref":"Guesswork.Constraint.Comparison.html#print/2"},{"type":"function","title":"Guesswork.Constraint.Comparison.reverse_constraint/1","doc":"","ref":"Guesswork.Constraint.Comparison.html#reverse_constraint/1"},{"type":"type","title":"Guesswork.Constraint.Comparison.t/0","doc":"","ref":"Guesswork.Constraint.Comparison.html#t:t/0"},{"type":"module","title":"Guesswork.Constraint.Result","doc":"A simple struct that indicates that a constraint has evaluated a to a new value.","ref":"Guesswork.Constraint.Result.html"},{"type":"type","title":"Guesswork.Constraint.Result.t/0","doc":"","ref":"Guesswork.Constraint.Result.html#t:t/0"},{"type":"module","title":"Guesswork.Constraint.Simple","doc":"A simple, uni-directional constraint built from a `Guesswork.Answer.Computation`.","ref":"Guesswork.Constraint.Simple.html"},{"type":"function","title":"Guesswork.Constraint.Simple.new/3","doc":"","ref":"Guesswork.Constraint.Simple.html#new/3"},{"type":"type","title":"Guesswork.Constraint.Simple.t/0","doc":"","ref":"Guesswork.Constraint.Simple.html#t:t/0"},{"type":"module","title":"Guesswork.Constraint.Store","doc":"The constraint store is just a trie that uses the input variables as keys to\nthe constraint, allowing for faster look up of constraints that can be resolved.\nNote that a single constraint can be listed multiple times since constraints\ncan have multiple 'directions' (if `a = b` then the constraint can be resolved\nby knowing either `a` or `b`).","ref":"Guesswork.Constraint.Store.html"},{"type":"module","title":"Stop Conditions - Guesswork.Constraint.Store","doc":"To allow stop conditions to be handled by this constraint store, the top level\nnode has a special child with the key stop_conditions.\nShould any constraint under that child return `:error` the store converts it to\n`:halt`.\nTo ensure this happens before a conflict is created in the answer set, when\nsearching the tree stop_conditions is always searched before other children.","ref":"Guesswork.Constraint.Store.html#module-stop-conditions"},{"type":"module","title":"Plural Results - Guesswork.Constraint.Store","doc":"When a constraint returns a plural result the constraint needs to be kept for a\nmore exact result.\nTo handle this, all constraints are stored with the last result, which  is checked.\nIf the two values are equal, the search continues.\nIf they are not, the result is returned but the constraint is retained if the\nreturned value is plural.","ref":"Guesswork.Constraint.Store.html#module-plural-results"},{"type":"module","title":"Memory Performance - Guesswork.Constraint.Store","doc":"This data structure prizes search effiency over memory\nefficiency, meaning passing tests (constraints that evaluate to `:ok`) that could\nbe dropped sometimes are not when it is easier to just keep searching the tree.","ref":"Guesswork.Constraint.Store.html#module-memory-performance"},{"type":"type","title":"Guesswork.Constraint.Store.constraint_node/0","doc":"","ref":"Guesswork.Constraint.Store.html#t:constraint_node/0"},{"type":"function","title":"Guesswork.Constraint.Store.constraints/1","doc":"Pulls all stored constraints.","ref":"Guesswork.Constraint.Store.html#constraints/1"},{"type":"function","title":"Guesswork.Constraint.Store.describe/1","doc":"Pulls all valid descriptions of the containted constraints.","ref":"Guesswork.Constraint.Store.html#describe/1"},{"type":"function","title":"Guesswork.Constraint.Store.eval_first/3","doc":"Takes the constraint store and an env, then returns the result of the first valid\nconstraint reached.","ref":"Guesswork.Constraint.Store.html#eval_first/3"},{"type":"function","title":"Guesswork.Constraint.Store.insert/2","doc":"","ref":"Guesswork.Constraint.Store.html#insert/2"},{"type":"function","title":"Guesswork.Constraint.Store.insert_stop_condition/2","doc":"","ref":"Guesswork.Constraint.Store.html#insert_stop_condition/2"},{"type":"type","title":"Guesswork.Constraint.Store.key/0","doc":"","ref":"Guesswork.Constraint.Store.html#t:key/0"},{"type":"function","title":"Guesswork.Constraint.Store.merge/2","doc":"Merges two constraint stores.","ref":"Guesswork.Constraint.Store.html#merge/2"},{"type":"function","title":"Guesswork.Constraint.Store.new/0","doc":"","ref":"Guesswork.Constraint.Store.html#new/0"},{"type":"function","title":"Guesswork.Constraint.Store.remove_inputs/2","doc":"Removes all inputs that are not in the supplied set.","ref":"Guesswork.Constraint.Store.html#remove_inputs/2"},{"type":"function","title":"Guesswork.Constraint.Store.remove_resolvable_constraints/2","doc":"Walks the trie removing any constraints for which all inputs are in the env and\nnot plural.\nNo constraints are resolved, only removed.","ref":"Guesswork.Constraint.Store.html#remove_resolvable_constraints/2"},{"type":"type","title":"Guesswork.Constraint.Store.t/0","doc":"","ref":"Guesswork.Constraint.Store.html#t:t/0"},{"type":"function","title":"Guesswork.Constraint.Store.variables/1","doc":"Returns all variables that the store is aware of.\nMay return duplicates.","ref":"Guesswork.Constraint.Store.html#variables/1"},{"type":"module","title":"Guesswork.EmptyCollection","doc":"A collection with no facts, falsehoods, or rules.","ref":"Guesswork.EmptyCollection.html"},{"type":"function","title":"Guesswork.EmptyCollection.store/0","doc":"","ref":"Guesswork.EmptyCollection.html#store/0"},{"type":"behaviour","title":"Guesswork.KnowledgeBase","doc":"Represents a querable knowledge base.","ref":"Guesswork.KnowledgeBase.html"},{"type":"callback","title":"Guesswork.KnowledgeBase.get_possible_facts/1","doc":"Pulls all facts could match the supplied fact.\n\nThe only requirement is that all true matches be returned, so theoretically\nall the facts could simply be dumped by the this function.","ref":"Guesswork.KnowledgeBase.html#c:get_possible_facts/1"},{"type":"callback","title":"Guesswork.KnowledgeBase.get_possible_falsehoods/1","doc":"Pulls all falsehoods could match the supplied fact.\n\nThe only requirement is that all true matches be returned, so theoretically\nall the falsehoods could simply be dumped by the this function.","ref":"Guesswork.KnowledgeBase.html#c:get_possible_falsehoods/1"},{"type":"callback","title":"Guesswork.KnowledgeBase.get_possible_rules/1","doc":"Pulls all rules could match the supplied fact.\n\nThe only requirement is that all true matches be returned, so theoretically\nall the rules could simply be dumped by the this function.","ref":"Guesswork.KnowledgeBase.html#c:get_possible_rules/1"},{"type":"module","title":"Guesswork.KnowledgeBase.Collection","doc":"Allows for the easy creation of `Guesswork.KnowledgeBase` by using the `deffact/1`,\n`deffalsehood/1`, and `defrule/2` macros in modules that invoke\n`use Guesswork.KnowledgeBase.Collection`.\n\nCurrently, everything is stored in the module's functions using a\n`Guesswork.KnowledgeBase.Collection.Store` that is created at compile time.\nAs such, the collection is not mutable.","ref":"Guesswork.KnowledgeBase.Collection.html"},{"type":"macro","title":"Guesswork.KnowledgeBase.Collection.deffact/1","doc":"Adds a new `Guesswork.Ast.Fact` to the collection as a true fact using the\nsupplied arguments.\nAll arguments must concrete.","ref":"Guesswork.KnowledgeBase.Collection.html#deffact/1"},{"type":"macro","title":"Guesswork.KnowledgeBase.Collection.deffalsehood/1","doc":"Adds a new `Guesswork.Ast.Fact` to the collection as a falsehood using the\nsupplied arguments.\nAll arguments must concrete.","ref":"Guesswork.KnowledgeBase.Collection.html#deffalsehood/1"},{"type":"macro","title":"Guesswork.KnowledgeBase.Collection.defrule/2","doc":"Adds a new `Guesswork.Ast.Rule` to the collection.\n\nMultiple caluses are joined with `Guesswork.Ast.And`.","ref":"Guesswork.KnowledgeBase.Collection.html#defrule/2"},{"type":"module","title":"Guesswork.KnowledgeBase.Collection.Store","doc":"Represents a very basic knowledge base with all rules and facts stored in memory.","ref":"Guesswork.KnowledgeBase.Collection.Store.html"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.get_possible_facts/2","doc":"Pulls all facts that match the relation name and arity of the supplied fact.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#get_possible_facts/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.get_possible_falsehoods/2","doc":"Pulls all falsehoods that match the relation name and arity of the supplied fact.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#get_possible_falsehoods/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.get_possible_rules/2","doc":"Pulls all rules who's headers match the relation name and arity of the\nsupplied fact.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#get_possible_rules/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.insert_fact/2","doc":"Inserts a fact into the knowledge base.\nIt must be concrete.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#insert_fact/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.insert_falsehood/2","doc":"Inserts a falsehood into the knowledge base.\nIt must be concrete.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#insert_falsehood/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.insert_rule/2","doc":"Inserts a rule into the knowledge base.","ref":"Guesswork.KnowledgeBase.Collection.Store.html#insert_rule/2"},{"type":"function","title":"Guesswork.KnowledgeBase.Collection.Store.new/0","doc":"","ref":"Guesswork.KnowledgeBase.Collection.Store.html#new/0"},{"type":"type","title":"Guesswork.KnowledgeBase.Collection.Store.t/0","doc":"","ref":"Guesswork.KnowledgeBase.Collection.Store.html#t:t/0"},{"type":"module","title":"Guesswork.Query","doc":"Represents an un-resolved query.","ref":"Guesswork.Query.html"},{"type":"function","title":"Guesswork.Query.new/2","doc":"Builds a new query.\n\nAvailable Options:\n\n* `:label` - Traverses plural entities looking for a specific answer sets.\n  If a list of strings is provided only the supplied variables are traversed. The default value is `false`.\n\n* `:knowledge_base` (`t:term/0`) - The `Guesswork.KnowledgeBase` used to resolve queries. The default value is `Guesswork.EmptyCollection`.\n\n* `:query_id` (`t:String.t/0`) - The id used to report telemetry. Defaults to a new `UUID` (v4).\n\n* `:precompute_count` (`t:integer/0`) - The number of answer sets to compute when resolving a fact.\n  This allows repeated facts (that would result in the same\n  matches) to reuse computation.\n  Note that, with recusive queries, this can lead to infinate searches. The default value is `0`.","ref":"Guesswork.Query.html#new/2"},{"type":"type","title":"Guesswork.Query.opts_list/0","doc":"","ref":"Guesswork.Query.html#t:opts_list/0"},{"type":"function","title":"Guesswork.Query.opts_schema/0","doc":"","ref":"Guesswork.Query.html#opts_schema/0"},{"type":"function","title":"Guesswork.Query.run/2","doc":"Takes a query and attempts to resolve `n` answers from it.","ref":"Guesswork.Query.html#run/2"},{"type":"type","title":"Guesswork.Query.t/0","doc":"","ref":"Guesswork.Query.html#t:t/0"},{"type":"module","title":"Guesswork.Telemetry","doc":"Used to track queries and their metrics.","ref":"Guesswork.Telemetry.html"},{"type":"function","title":"Guesswork.Telemetry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Guesswork.Telemetry.html#child_spec/1"},{"type":"function","title":"Guesswork.Telemetry.fail_run/5","doc":"Indicates that a query run has crashed.\n\nNote that the `duration` is assumed to be the native time from a telemetry event.","ref":"Guesswork.Telemetry.html#fail_run/5"},{"type":"function","title":"Guesswork.Telemetry.fail_span/5","doc":"Indicates that a query run has crashed.\n\nNote that the `duration` is assumed to be the native time from a telemetry event.","ref":"Guesswork.Telemetry.html#fail_span/5"},{"type":"function","title":"Guesswork.Telemetry.get_query_metrics/2","doc":"Pulls all metrics for the requested `query_id`.","ref":"Guesswork.Telemetry.html#get_query_metrics/2"},{"type":"function","title":"Guesswork.Telemetry.increment_assignments/3","doc":"Increases the number of assignments tracked for a query.","ref":"Guesswork.Telemetry.html#increment_assignments/3"},{"type":"function","title":"Guesswork.Telemetry.increment_precomputed_streams/3","doc":"Increases the number of precomputed streams tracked for a query.","ref":"Guesswork.Telemetry.html#increment_precomputed_streams/3"},{"type":"function","title":"Guesswork.Telemetry.increment_substitutions/3","doc":"Increases the number of substitutions tracked for a query.","ref":"Guesswork.Telemetry.html#increment_substitutions/3"},{"type":"function","title":"Guesswork.Telemetry.increment_tests/3","doc":"Increases the number of test computations tracked for a query.","ref":"Guesswork.Telemetry.html#increment_tests/3"},{"type":"function","title":"Guesswork.Telemetry.list_query_runs/3","doc":"Pulls all query runs for a specific or all queries from the system.\n\nAvailable Options:\n* `:page_size` (`t:integer/0`) - The number of items to pull per page. The default value is `100`.\n\n* `:page_token` (`t:term/0`) - The continuation used to get the next page.","ref":"Guesswork.Telemetry.html#list_query_runs/3"},{"type":"function","title":"Guesswork.Telemetry.list_query_spans/3","doc":"Pulls all spans for a specific query.\n\nAvailable Options:\n* `:page_size` (`t:integer/0`) - The number of items to pull per page. The default value is `100`.\n\n* `:page_token` (`t:term/0`) - The continuation used to get the next page.","ref":"Guesswork.Telemetry.html#list_query_spans/3"},{"type":"type","title":"Guesswork.Telemetry.pagination_opts/0","doc":"","ref":"Guesswork.Telemetry.html#t:pagination_opts/0"},{"type":"function","title":"Guesswork.Telemetry.start_link/1","doc":"","ref":"Guesswork.Telemetry.html#start_link/1"},{"type":"function","title":"Guesswork.Telemetry.start_run/4","doc":"Indicates that a query run has started.\nIf the run has already been started the insert will fail, but quietly.\n\nNote that the `start_time` is assumed to be the native time from a telemetry\nevent.","ref":"Guesswork.Telemetry.html#start_run/4"},{"type":"function","title":"Guesswork.Telemetry.start_span/5","doc":"Indicates that a query span has started.\nIf the span has already been started the insert will fail, but quietly.\n\nNote that the `start_time` is assumed to be the native time from a telemetry\nevent.","ref":"Guesswork.Telemetry.html#start_span/5"},{"type":"function","title":"Guesswork.Telemetry.stop_run/4","doc":"Indicates that a query run has stopped.\n\nNote that the `duration` is assumed to be the native time from a telemetry event.","ref":"Guesswork.Telemetry.html#stop_run/4"},{"type":"function","title":"Guesswork.Telemetry.stop_span/5","doc":"Indicates that a query span has stopped.\n\nNote that the `duration` is assumed to be the native time from a telemetry event.","ref":"Guesswork.Telemetry.html#stop_span/5"},{"type":"module","title":"Guesswork.Telemetry.EventHandler","doc":"Holds event handlers for telemetry events.","ref":"Guesswork.Telemetry.EventHandler.html"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.assignment_event_name/0","doc":"Returns the event name used for reporting when simple assignments happen.","ref":"Guesswork.Telemetry.EventHandler.html#assignment_event_name/0"},{"type":"macro","title":"Guesswork.Telemetry.EventHandler.build_span_handlers/2","doc":"","ref":"Guesswork.Telemetry.EventHandler.html#build_span_handlers/2"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.computation_event_name/0","doc":"Returns the event name used for reporting when a compuations (test or assignment)\ncompletes successfully.","ref":"Guesswork.Telemetry.EventHandler.html#computation_event_name/0"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.fact_match_event_name/0","doc":"Returns the event name use for reporting when fact matches are calculated.","ref":"Guesswork.Telemetry.EventHandler.html#fact_match_event_name/0"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.handle_event/4","doc":"","ref":"Guesswork.Telemetry.EventHandler.html#handle_event/4"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.handled_events/0","doc":"Returns all telemetry events that `handle_event/4` is set up to handle.","ref":"Guesswork.Telemetry.EventHandler.html#handled_events/0"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.precompute_event_name/0","doc":"Returns the event name used for reporting when streams are precomputed.","ref":"Guesswork.Telemetry.EventHandler.html#precompute_event_name/0"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.resolve_span_prefix/0","doc":"Returns the event prefix used for query spans.","ref":"Guesswork.Telemetry.EventHandler.html#resolve_span_prefix/0"},{"type":"function","title":"Guesswork.Telemetry.EventHandler.union_span_prefix/0","doc":"Returns the event prefix used for union spans (`Guesswork.Ast.And.Statement.resolve/2`).","ref":"Guesswork.Telemetry.EventHandler.html#union_span_prefix/0"},{"type":"module","title":"Guesswork.Telemetry.PaginatedResponse","doc":"A paginated response, along with the token needed pull the next page.\n\nNote that a non-nil `next_page_token` does not mean there is certain to be\nanother page.\nIn the case where the page size evenly divides the total number of items you\nwill have one empty page at the end.","ref":"Guesswork.Telemetry.PaginatedResponse.html"},{"type":"function","title":"Guesswork.Telemetry.PaginatedResponse.new/2","doc":"","ref":"Guesswork.Telemetry.PaginatedResponse.html#new/2"},{"type":"type","title":"Guesswork.Telemetry.PaginatedResponse.t/1","doc":"","ref":"Guesswork.Telemetry.PaginatedResponse.html#t:t/1"},{"type":"module","title":"Guesswork.Telemetry.QueryMetrics","doc":"Represents the tracked actions a query can take:\n\n- `assignments` - Happens during a `Guesswork.Ast.Assign`, `Guesswork.Ast.OneOf`,\n  `Guesswork.Ast.Is`, or `Guesswork.Ast.Fact`.\n  Simply represents when a variable gets assigned a possible entity value.\n\n- `substitutions` - This only can happen during a `Guesswork.Ast.Fact` and\n  represents when a rule is found to match a fact statement, resulting in a\n  substitution.\n\n- `tests` - These are the result of a `Guesswork.Ast.Is` that checks against a\n  concrete entity.\n\n- `precomputed_streams` - The number of streams that `Guesswork.Ast.Precomputed`\n  groups together.","ref":"Guesswork.Telemetry.QueryMetrics.html"},{"type":"function","title":"Guesswork.Telemetry.QueryMetrics.new/5","doc":"","ref":"Guesswork.Telemetry.QueryMetrics.html#new/5"},{"type":"type","title":"Guesswork.Telemetry.QueryMetrics.t/0","doc":"","ref":"Guesswork.Telemetry.QueryMetrics.html#t:t/0"},{"type":"module","title":"Guesswork.Telemetry.QueryRun","doc":"Represents a single 'run', or unification, of a query.","ref":"Guesswork.Telemetry.QueryRun.html"},{"type":"function","title":"Guesswork.Telemetry.QueryRun.ets_key/1","doc":"Returns a key used for storage in ets.","ref":"Guesswork.Telemetry.QueryRun.html#ets_key/1"},{"type":"function","title":"Guesswork.Telemetry.QueryRun.finish/2","doc":"Takes a query run and the `duration` of the run then completes the record by\ncalculating and inserting the `end_time` and `duration`.\n`duration` is assumed to be `:native` unix times.","ref":"Guesswork.Telemetry.QueryRun.html#finish/2"},{"type":"function","title":"Guesswork.Telemetry.QueryRun.finish/3","doc":"Takes a query run the `duration` and the error that failed the query, and then\ncompletes the record by calculating and inserting the `end_time` and `duration`.\n`duration` is assumed to be `:native` unix times.","ref":"Guesswork.Telemetry.QueryRun.html#finish/3"},{"type":"function","title":"Guesswork.Telemetry.QueryRun.new/3","doc":"Attempts to build a new query run entry using the needed ids and the start time\nprovided by telemetry (assumes a `:native` unix time).","ref":"Guesswork.Telemetry.QueryRun.html#new/3"},{"type":"type","title":"Guesswork.Telemetry.QueryRun.t/0","doc":"","ref":"Guesswork.Telemetry.QueryRun.html#t:t/0"},{"type":"module","title":"Guesswork.Telemetry.QuerySpan","doc":"Represents a span within a query run.","ref":"Guesswork.Telemetry.QuerySpan.html"},{"type":"function","title":"Guesswork.Telemetry.QuerySpan.ets_key/1","doc":"Returns a key used for storage in ets.","ref":"Guesswork.Telemetry.QuerySpan.html#ets_key/1"},{"type":"function","title":"Guesswork.Telemetry.QuerySpan.fail/3","doc":"Takes a query span, the `duration` of the span, and the final error, then\ncompletes the record by calculating and inserting the `end_time` and `duration`.\n`duration` is assumed to be `:native` unix times.","ref":"Guesswork.Telemetry.QuerySpan.html#fail/3"},{"type":"function","title":"Guesswork.Telemetry.QuerySpan.finish/3","doc":"Takes a query span, the `duration` of the span, and the final metadata, then\ncompletes the record by calculating and inserting the `end_time` and `duration`.\n`duration` is assumed to be `:native` unix times.","ref":"Guesswork.Telemetry.QuerySpan.html#finish/3"},{"type":"type","title":"Guesswork.Telemetry.QuerySpan.metadata/0","doc":"","ref":"Guesswork.Telemetry.QuerySpan.html#t:metadata/0"},{"type":"function","title":"Guesswork.Telemetry.QuerySpan.new/4","doc":"Attempts to build a new query run entry using the needed id, context, type, and\nthe start time provided by telemetry (assumes a `:native` unix time).","ref":"Guesswork.Telemetry.QuerySpan.html#new/4"},{"type":"type","title":"Guesswork.Telemetry.QuerySpan.t/0","doc":"","ref":"Guesswork.Telemetry.QuerySpan.html#t:t/0"},{"type":"module","title":"Schedule","doc":"","ref":"Schedule.html"},{"type":"function","title":"Schedule.count_shifts/2","doc":"","ref":"Schedule.html#count_shifts/2"},{"type":"function","title":"Schedule.new/7","doc":"","ref":"Schedule.html#new/7"},{"type":"function","title":"Schedule.to_list/1","doc":"","ref":"Schedule.html#to_list/1"},{"type":"module","title":"Shift","doc":"","ref":"Shift.html"},{"type":"function","title":"Shift.member?/2","doc":"","ref":"Shift.html#member?/2"},{"type":"function","title":"Shift.new/3","doc":"","ref":"Shift.html#new/3"},{"type":"function","title":"Shift.to_kino_row/2","doc":"","ref":"Shift.html#to_kino_row/2"},{"type":"function","title":"Shift.valid?/1","doc":"","ref":"Shift.html#valid?/1"},{"type":"extras","title":"Guesswork","doc":"# Guesswork\n\nGuesswork is a logic programming library for Elixir.\nIt is heavily inspired by Prolog, but attempts to use idiomatic Elixir when\nexpressing problems and their solutions.","ref":"readme.html"},{"type":"extras","title":"Versions - Guesswork","doc":"This project is tested on the following elixir/OTP versions:\n- erlang 26.2.5\n- elixir 1.17.3","ref":"readme.html#versions"},{"type":"extras","title":"Examples - Guesswork","doc":"All Examples are meant to be run with [LiveBook](https://livebook.dev/) and can\nbe found in the `examples/` directory.","ref":"readme.html#examples"},{"type":"extras","title":"Design - Guesswork","doc":"Guesswork is designed to have two stages when solving problems: resolution and\nunification.\nDuring resolution statements (`Guesswork.Ast.Statement`) are turned into streams\nwhich, once run and turned into a `Guesswork.Answer.Result`, unify possible\nsolutions looking for valid ones.","ref":"readme.html#design"},{"type":"extras","title":"Benchmarks - Guesswork","doc":"All benchmarks are built with [Benchee](https://hexdocs.pm/benchee/readme.html),\ncan be found in the `benchmarks/` directory, and run with `make run_benchmarks`\n(this can take a while, other make targets exist for individual benchmarks).\nThe results can be found in the `benchmark_results/` directory.","ref":"readme.html#benchmarks"},{"type":"extras","title":"Future Work - Guesswork","doc":"Development work is focused on two things, making the system more ergonomic,\nperformant, and working through logic programming puzzles to figure out where the\nlibrary is deficient.\nThis list should not be considered a road map to 1.0.\n\n- Constraint Logic Programming\n- 8/N Queens Problem\n- ~~Scheduling Problems~~\n- Sigil for building statements\n- ~~Adding benchmarks~~","ref":"readme.html#future-work"},{"type":"extras","title":"N Factorial","doc":"# N Factorial\n\n```elixir\nMix.install([\n  {:guesswork, \"~> 0.7\"}  \n])\n```","ref":"n_factorial.html"},{"type":"extras","title":"Recursive Queries - N Factorial","doc":"```elixir\nimport Guesswork.Ast\n\nalias Guesswork.Ast.And\nalias Guesswork.Ast.Fact\nalias Guesswork.Ast.Is\n```\n\nLooking for a factorial is a little different from some of the other queries \nwe have written.\nInstead generating possible values with criteria, we'll use a recursive query \nto define what a valid factorial is.\nIf you look below you can see some of the normal things you would see in a recursive\ndefinition: `deffact` is used to define a base case, and `defrule` is used to define\nthe recursive step (where `Fact` invoces the next step).\n\n```elixir\ndefmodule FactorialKB do\n  use Guesswork.KnowledgeBase.Collection\n\n  deffact n_factorial(0, 1)\n  defrule n_factorial(n, f) do\n    Is.new(true, [n, 0], &Kernel.>/2, halt_on_error: true)\n    Is.new(true, [f, f1], &Kernel.>=/2, halt_on_error: true)\n    Is.new(n, [n1, 1], &Kernel.+/2)\n    Is.new(n1, [n, 1], &Kernel.-/2)\n    Is.new(f, [n, f1], &Kernel.*/2)\n    Fact.new(:n_factorial, [n1, f1])\n  end\nend\n```\n\nThis can be used simillarly to other queries, like the pythagorean triple query, where \nwe can pull the first `n` valid answer sets.\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [n, f])), 10, knowledge_base: FactorialKB)\n```\n\nBut because the query is defined in terms of `n` and `f` we can query what `f` is \nwhen `n` is `5`.\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [5, f])), 10, knowledge_base: FactorialKB)\n```\n\nAnd the system supports binding `f` and searching for `n`.\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [n, 120])), 10, knowledge_base: FactorialKB)\n```\n\nEven when there are multiple valid values for `n`, the query will return them all \n(provided you ask for them).\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [n, 1])), 10, knowledge_base: FactorialKB)\n```","ref":"n_factorial.html#recursive-queries"},{"type":"extras","title":"Stop Conditions - N Factorial","doc":"If you look above, you'll notice that even though we ask for `10` results \nfrom `Guesswork.query/3`, we only get the valid values, and then the query \nterminates.\nThis is done using stop conditions, via the `:halt_on_error` option of `Is`.\n\nStop conditions can also allow us to use the query to search for potentially\ninvalid values.\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [n, 121])), 10, knowledge_base: FactorialKB)\n```\n\nOr test if a relationship is valid between some `n` and `f`.\n\n```elixir\nGuesswork.query(term(Fact.new(:n_factorial, [4, 120])), 10, knowledge_base: FactorialKB)\n```\n\nStop conditions will terminate the entire stream after they are triggered \n(unlike normal tests, which will just disqualify individual answer sets).\nBut when we didn't define anything for `n` or `f`, we never triggered the stop conditions,\nthe query just kept returning values.\n\nThis is because the stop condition needs to be defined in terms of some concrete value.\nYou you can think of the recursive definition as building possible answer sets, first \nwith just the base fact, then with one recursive, then two, and so on.\nThis is why the query is resonably fast, despite the large numbers; it doesn't have to \nsearch all possible numbers, just instantiate and resolve the answer sets.\n\nBut when you define `n` or `f` you need to account for the possibility that there is no\npossible valid answer set, and tell the query to stop looking.\nIn this case we just have the system stop once it fines a recursive step with values \ngreater than the values we are looking for, since the following anwser sets (which will \nall have more recursive steps) will have increasingly larger values, and therefore \ncannot be valid either.\n\nIn the unbound query, because `n` and `f` are always products of the answer set (and \nnot defined by us) the stop condition cannot possibly be true.","ref":"n_factorial.html#stop-conditions"},{"type":"extras","title":"Telemetry Data","doc":"# Telemetry Data\n\n```elixir\nMix.install(\n  [\n    {:guesswork, \"~> 0.7\"},\n    {:kino, \"~> 0.13\"},\n    {:kino_explorer, \"~> 0.1.23\"}\n  ])\n```","ref":"telemetry.html"},{"type":"extras","title":"Collecting Telemetry Data - Telemetry Data","doc":"```elixir\nimport Guesswork.Ast\n\nalias Guesswork.Ast.And\nalias Guesswork.Ast.Assign\nalias Guesswork.Ast.OneOf\nalias Guesswork.Answer.Result\nalias Guesswork.Query\nalias Guesswork.Telemetry\nalias Guesswork.Telemetry.QuerySpan\nalias Guesswork.Telemetry.QueryRun\n\nrequire Explorer.DataFrame, as: DF\n```\n\nGuesswork provides data about your queries via `:telemetry` events and spans so that \nyou can figure out what is slowing a query down.\nIf you would like to hook into the events yourself, take a look at the\n`Guesswork.Telemetry.EventHandler` module, but a good place to start is just\nby looking at the data collected by `Guesswork.Telemetry`.\n\nStart by starting up the telemetry `GenServer`.\n\n```elixir\n{:ok, pid} = Telemetry.start_link()\n```\n\nThen, we'll rerun the pythagorean triples queries, and see what data we've collected.\n\nYou'll notice that these queries take noticibly longer.\nRunning queries with telemetry is expensive (you'll see why below), and should\nmostly be used for debugging.\n\n```elixir\nquery =\n  Query.new(\n    term(\n      And.new([\n        OneOf.new(a, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(b, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(c, Stream.iterate(1, &(&1 + 1))),\n        is(true, fn a, b -> a   Integer.pow(c, 2) - Integer.pow(a, 2) - Integer.pow(b, 2) end)\n      ])\n    )\n  )\n\nResult.run(query, 10)\nResult.run(query, 100)\n```\n\nTo pull the data we use the `list_query_runs/3` function.\nThe first thing to note is the ids: `query_id` and `run_id`.\n`query_id` is assigned when the query is created by `Guesswork.query.new/2`,\nand propagated to all the statements within it.\nAs a result of this implementation, all metrics and spans are for the\nquery, not the run, which has a unique `run_id` per entire result\nset.\n\nNote here that we're using an `Explorer.DataFrame` for all of our analysis\n(and it has a great integration with Livebook).\n\n```elixir\nruns = Telemetry.list_query_runs(pid).values\n  |> Enum.map(&Map.from_struct/1)\n  |> DF.new()\n```\n\nOnce we have the `query_id` (this also could have been pulled from\nthe `query` struct) we pull the metrics for both runs.\nThe metrics show about 600, and over 6.2 million tests run.\nThis makes a lot of sense given the above query is so brute force, and must\nrun so many test to remove possible combinations of `a`, `b`, and `c`.\n\n```elixir\nquery_id = runs[\"query_id\"][0]\nTelemetry.get_query_metrics(pid, query_id)\n```\n\nThe last kind of telemetry data `Guesswork` tracks are spans; currently\njust spans on calculated unions (`Guesswork.Ast.And`).\nThis is the primary action taken, because it combines possible values.\nYou should note that most of these spans report no results (`unions_calculated`).\nThis is in line with the number of tests we have to run to get those\n110 results.\n\n```elixir\nTelemetry.list_query_spans(pid, query_id, page_size: 1000).values\n|> Enum.map(&Map.from_struct/1)\n|> Enum.map(&Map.delete(&1, :ctx))\n|> Enum.map(fn row -> Map.update(row, :type, \"N/A\", &Atom.to_string/1) end)\n|> DF.new()\n|> DF.unnest(:metadata)\n```","ref":"telemetry.html#collecting-telemetry-data"},{"type":"extras","title":"Improving Queries - Telemetry Data","doc":"Now that we're collecting this data, lets see if its possible to improve\nthe performance of our query.\nYou'll note that that first test (`is(true, fn a, b -> a   a   a and c > b end),\n        is(0, fn a, b, c -> Integer.pow(c, 2) - Integer.pow(a, 2) - Integer.pow(b, 2) end)\n      ])\n    )\n  )\n\nResult.run(query, 10)\nResult.run(query, 100)\n```\n\nThis this worse, much worse.\nAll the times are below.\n\n```elixir\nTelemetry.list_query_runs(pid).values\n|> Enum.map(&Map.from_struct/1)\n|> DF.new()\n```\n\nTo try understand why, lets take a look at the metrics for our new query.\nYou should notice that it took the exact same number of assignments (we\njust have to try that many options to find our answer set), but way more\ntests!\n\nThis is a side effect of how `Guesswork.Ast.And` operates.\nAs it calculates unions (combining possible answer sets) it removes invalid\nanswer sets at each iteration.\nYou should also note that unions grow exponentially and our new test requires\nthree variables to assigned instead of two.\nThis means that the so by the test must be run later and by time you can run\nthe test, there are just more answer sets to filter out!\n(It is worth pointing out that the tests themselves are run less often, but \nthat more computation is still happening here.)\n\n```elixir\nTelemetry.get_query_metrics(pid, query_id)\n```\n\nUsing this information, we can guess that its better to split that `is`\nstatement up.\n\n```elixir\n%Query{id: query_id} = query =\n  Query.new(\n    term(\n      And.new([\n        OneOf.new(a, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(b, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(c, Stream.iterate(1, &(&1 + 1))),\n        is(true, fn a, b -> a   c > b end),\n        is(0, fn a, b, c -> Integer.pow(c, 2) - Integer.pow(a, 2) - Integer.pow(b, 2) end)\n      ])\n    )\n  )\n\nResult.run(query, 10)\nResult.run(query, 100)\n```\n\nThis did much better.\nWe can see that it takes a little under 70% of the time of our first query,\nand it is almost twice as fast our second query!\n\n```elixir\nTelemetry.list_query_runs(pid).values\n|> Enum.map(&Map.from_struct/1)\n|> DF.new()\n```\n\nFinally, we can confirm that, indeed, this query ran many fewer tests.\n\n```elixir\nTelemetry.get_query_metrics(pid, query_id)\n```","ref":"telemetry.html#improving-queries"},{"type":"extras","title":"Family Tree","doc":"# Family Tree\n\n```elixir\nMix.install([\n  {:guesswork, \"~> 0.7\"},\n  {:kino, \"~> 0.13\"}\n])\n```","ref":"familly_tree.html"},{"type":"extras","title":"Build a Knowledge Base - Family Tree","doc":"Describing family trees is often used as the 'Hello World' of logical programming,\nso this example will show you how to build a Knowledge Base and query it.\n\n```elixir\nimport Guesswork.Ast\n\nalias Guesswork.Ast.And\nalias Guesswork.Ast.Fact\nalias Guesswork.Answer.Result\n```\n\nStart by adding rules and concrete facts to the knowledge base, in this case parent \nchild relationships and grandparent realtionships defined as parent relationships.\n\n```elixir\ndefmodule KB do\n  use Guesswork.KnowledgeBase.Collection\n\n  deffact parent(:bob, :joe)\n  deffact parent(:bob, :jean)\n  deffact parent(:mary, :joe)\n  deffact parent(:bobby, :jean)\n\n  deffact parent(:christy, :bob)\n  deffact parent(:lance, :bob)\n\n  deffact parent(:billy, :christy)\n  deffact parent(:meg, :lanceb)\n\n  defrule grandparent(gp, gc) do \n    Fact.new(:parent, [gp, p])\n    Fact.new(:parent, [p, gc])\n  end\nend\n```","ref":"familly_tree.html#build-a-knowledge-base"},{"type":"extras","title":"Query the System - Family Tree","doc":"Now you can query the system.\nThis first query is simplier since it only asks questions of the\nconcrete facts stored in the knowledge base; in this case what are\nbob's children.\n\n```elixir\nGuesswork.query(term(Fact.new(:parent, [:bob, child])), 10, knowledge_base: KB)\n```\n\nYou can also ask more complex questions, like what are all the grandparents\nof joe.\nYou should note that the variable internal to the grandparent rule (`p`) is not\npart of the final answer set.\nThis is because `Guesswork` respects lexical scoping of variables.\n\n```elixir\nGuesswork.query(term(Fact.new(:grandparent, [gp, :joe])), 100, knowledge_base: KB)\n```","ref":"familly_tree.html#query-the-system"},{"type":"extras","title":"Shift Scheduling","doc":"# Shift Scheduling\n\n```elixir\nMix.install(\n  [\n    {:guesswork, \"~> 0.8\"}, \n    {:kino, \"~> 0.14\"},\n    {:kino_explorer, \"~> 0.1.23\"}\n  ], consolidate_protocols: false)\n```","ref":"shift_scheduling.html"},{"type":"extras","title":"Shift Scheduling - Shift Scheduling","doc":"```elixir\nimport Guesswork.Ast\n\nrequire Explorer.DataFrame, as: DF\n\nalias Guesswork.Ast.And\nalias Guesswork.Ast.Fact\nalias Guesswork.Ast.Assign\nalias Guesswork.Ast.OneOf\nalias Guesswork.Ast.Variable\nalias Guesswork.Ast.Is\nalias Guesswork.Answer.Result\n```\n\nShift Scheduling is another interesting use case for logic programming.\nIt fits well since constraints are easily expressed in logic.\nFirst we'll set up some basic data types to help run tests and display\nthe result using `Kino`.\nThis will clean up our logical statements by pushing the more standard elixir\nstatements out of the queries and into these modules.\n\n```elixir\ndefmodule Shift do\n  @derive [Guesswork.Ast.Entity]\n  defstruct [:bartender, :barback, :waiter]\n\n  def new(bartender, barback, waiter) do\n    %__MODULE__{bartender: bartender, barback: barback, waiter: waiter}\n  end\n\n  def valid?(%__MODULE__{bartender: bartender, barback: barback, waiter: waiter}) do\n    bartender != barback and bartender != waiter and barback != waiter\n  end\n\n  def member?(%__MODULE__{bartender: bartender, barback: barback, waiter: waiter}, value) do\n    Enum.member?([bartender, barback, waiter], value)\n  end\n\n  def to_kino_row(shift, day) do\n    Map.merge(Map.from_struct(shift), %{day: day})\n  end\nend\n\ndefmodule Schedule do\n  @derive [Guesswork.Ast.Entity]\n  defstruct [:monday, :tuesday, :wednesday, :thursday, :friday, :saturday, :sunday]\n\n  def new(monday, tuesday, wednesday, thursday, friday, saturday, sunday) do\n    %__MODULE__{\n      monday: monday,\n      tuesday: tuesday,\n      wednesday: wednesday,\n      thursday: thursday,\n      friday: friday,\n      saturday: saturday,\n      sunday: sunday,\n    }\n  end\n\n  def to_list(%__MODULE__{\n      monday: monday,\n      tuesday: tuesday,\n      wednesday: wednesday,\n      thursday: thursday,\n      friday: friday,\n      saturday: saturday,\n      sunday: sunday\n  }) do\n    [monday, tuesday, wednesday, thursday, friday, saturday, sunday]\n  end\n\n  def count_shifts(schedule, worker) do\n    to_list(schedule)\n    |> Enum.count(&Shift.member?(&1, worker))\n  end\n\n  defimpl Kino.Render do\n    def to_livebook(schedule) do\n      [:monday, :tuesday, :wednesday, :thursday, :friday, :saturday, :sunday]\n      |> Enum.zip_with(Schedule.to_list(schedule), &Shift.to_kino_row(&2, Atom.to_string(&1)))\n      |> Kino.DataTable.new(keys: [:day, :bartender, :barback, :waiter])\n      |> Kino.Render.to_livebook()\n    end\n  end\nend\n```\n\nHere is our first attempt at the shift scheduling query.\nFirst we precompute all the valid shifts using a `for` comprehension and a filter.\nThen we use `OneOf`s and `is`es to build the query.\nThere are just two things to note.\nFirst, is the use of the pin operator (`^`) to pull in a variable from outside the `term`.\nSecond, is how we are using `Enum.shuffle/1` to in the `OneOf`s.\n\nIf you try running the query without them it is much slower.\nThis is because the shifts need to be different on different days, so starting with\ndifferent lists of shift means we are more likely to hit shifts that don't conflict\nwith each other.\n\n```elixir\npossible_shifts = for bartender <- [\"bob\", \"paul\", \"jenny\", \"chris\"],\n  barback <- [\"bob\", \"milly\", \"joe\", \"paul\", \"jenny\"],\n  waiter <- [\"milly\", \"jenny\", \"bob\"] do\n   Shift.new(bartender, barback, waiter)\nend\n|> Enum.filter(&Shift.valid?/1)\n\nanswer = term(And.new([\n  # Assign shifts to days.\n  OneOf.new(monday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(tuesday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(wednesday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(thursday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(friday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(saturday, Enum.shuffle(^possible_shifts)),\n  OneOf.new(sunday, Enum.shuffle(^possible_shifts)),\n  # Build the schedule.\n  is(schedule, fn monday, tuesday, wednesday, thursday, friday, saturday, sunday ->\n    Schedule.new(monday, tuesday, wednesday, thursday, friday, saturday, sunday)\n  end),\n  # The days specific workers aren't available.\n  is(false, fn monday -> Shift.member?(monday, \"bob\") end),\n  is(false, fn monday -> Shift.member?(monday, \"chris\") end),\n  is(false, fn tuesday -> Shift.member?(tuesday, \"jenny\") end),\n  is(false, fn thursday -> Shift.member?(thursday, \"milly\") end),\n  is(false, fn saturday -> Shift.member?(saturday, \"bob\") end),\n  # The limits on the number of shifts each worker wants.\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"bob\") >= 4 end),\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"joe\") <= 2 end),\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"chris\") <= 4 end),\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"milly\") <= 3 end),\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"jenny\") >= 4 end),\n  is(true, fn schedule -> Schedule.count_shifts(schedule, \"paul\") <= 4 end),\n]))\n|> Guesswork.query(1)\n```\n\nFinally, We'll extract the schedule and use the `Kino` integration we built\npreviously to display it.\n\n```elixir\n{:bound, schedule} = answer.answer_sets\n|> List.first()\n|> get_in([\"schedule\"])\n\nschedule\n```","ref":"shift_scheduling.html#shift-scheduling"},{"type":"extras","title":"More Idomatic Queries - Shift Scheduling","doc":"Our last query works, but it isn't really ideal.\nInstead of building in a script, we should be using a `Guesswork.KnowledgeBase`\nto compile the information we need.\nFurthermore, instead of using the `for` comprehension we really should be using\n`Guesswork`'s own tools to calculate it, since building a valid shift is really\nthe same problem as building a valid schedule, just simpler.\n\nMost of the changes below ar in the `ShiftKB`.\nWe establish which workers can do what using `deffact`, and then we can build a\nrule that says a shift has one of each type of worker, and must statisfy\n`Shift.valid?/1`.\n\nThe final query ends up looking a lot like our first query, only we use\n`Guesswork.Ast.Fact`s to pull in the information the `ShiftKB` holds instead of\nusing the pin operator.\nIn fact, the query event acts very similarly to the first query; when `Guesswork`\nsees repeated `Guesswork.Ast.Fact`s, it pulls a cached version of the stream that \nhas been partially evaluated in advance.\nYou should note that this is not done by default since precomputation on recursive \nqueries can lead to infinate searches.\n\n```elixir\ndefmodule ShiftKB do\n  use Guesswork.KnowledgeBase.Collection\n\n  for worker <- [\"bob\", \"paul\", \"jenny\", \"chris\"] do\n    deffact bartender(^worker)\n  end\n\n  for worker <- [\"bob\", \"milly\", \"joe\", \"paul\", \"jenny\"] do\n    deffact barback(^worker)\n  end\n\n  for worker <- [\"bob\", \"milly\", \"jenny\"] do\n    deffact waiter(^worker)\n  end\n\n  defrule shift(shift) do\n    Fact.new(:bartender, [w1])\n    Fact.new(:barback, [w2])\n    Fact.new(:waiter, [w3])\n    Is.new(shift, [w1, w2, w3], &Shift.new/3)\n    Is.new(true, [shift], &Shift.valid?/1)\n  end\nend\n\nanswer = Guesswork.query(term(\n  And.new([\n    # Assign shifts to days.\n    Fact.new(:shift, [monday]),\n    Fact.new(:shift, [tuesday]),\n    Fact.new(:shift, [wednesday]),\n    Fact.new(:shift, [thursday]),\n    Fact.new(:shift, [friday]),\n    Fact.new(:shift, [saturday]),\n    Fact.new(:shift, [sunday]),\n    # Build the schedule.\n    is(schedule, fn monday, tuesday, wednesday, thursday, friday, saturday, sunday ->\n      Schedule.new(monday, tuesday, wednesday, thursday, friday, saturday, sunday)\n      end),\n    # The days specific workerss aren't available.\n    is(false, fn monday -> Shift.member?(monday, \"bob\") end),\n    is(false, fn monday -> Shift.member?(monday, \"chris\") end),\n    is(false, fn tuesday -> Shift.member?(tuesday, \"jenny\") end),\n    is(false, fn thursday -> Shift.member?(thursday, \"milly\") end),\n    is(false, fn saturday -> Shift.member?(saturday, \"bob\") end),\n    # The limits on the number of shifts each worker wants.\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"bob\") >= 4 end),\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"joe\") <= 2 end),\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"chris\") <= 4 end),\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"milly\") <= 3 end),\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"jenny\") >= 4 end),\n    is(true, fn schedule -> Schedule.count_shifts(schedule, \"paul\") <= 4 end),\n  ])\n), 1, knowledge_base: ShiftKB, precompute_count: 25)\n```\n\nFinally, we can again extract the schedule and see the displayed table.\n\n```elixir\n{:bound, schedule} = answer.answer_sets\n|> List.first()\n|> get_in([\"schedule\"])\n\nschedule\n```","ref":"shift_scheduling.html#more-idomatic-queries"},{"type":"extras","title":"Pythagorean Triples","doc":"# Pythagorean Triples\n\n```elixir\nMix.install([\n  {:guesswork, \"~> 0.7\"},\n  {:kino, \"~> 0.13\"}\n])\n```","ref":"pythagorean_triples.html"},{"type":"extras","title":"Queries that Calculate Values - Pythagorean Triples","doc":"```elixir\nimport Guesswork.Ast\n\nalias Guesswork.Ast.And\nalias Guesswork.Ast.Assign\nalias Guesswork.Ast.OneOf\nalias Guesswork.Answer.Result\nalias Guesswork.Query\n```\n\nQueries don't necisarily need facts to operate (in fact, `Guesswork.query/2` will just\nuse an empty collection), they can use assigned values, and then calculate new values.\nThe below example is a really simple query that uses `Guesswork.Ast.Assign` to assign\nvalues to `a` and  `b`, and then uses `is` to calculated a third value based on the first two.\n\nIt should be noted that, in most cases, statements are created with functions.\nBut the `is` macro is needed to build a `Guesswork.Ast.Is`.\n\n```elixir\nGuesswork.query(\n  term(And.new([Assign.new(a, 1), Assign.new(b, 2), is(d, fn a, b -> b - a end)])),\n  10\n)\n```\n\nBelow is a more complex example, calculating pythagorean triples.\nInstead of using `set` we use `one_of` to assign an enumerable to a variable, so that\nthe variable can be one of any of the values.\nThen we use `is` again, but this time to test that our variables really are pythagorean\ntriples.\n\n```elixir\nGuesswork.query(\n  term(\n    And.new([\n      OneOf.new(a, 1..10),\n      OneOf.new(b, 1..10),\n      OneOf.new(c, 1..10),\n      is(true, fn a, b -> a   Integer.pow(c, 2) - Integer.pow(a, 2) - Integer.pow(b, 2) end)\n    ])\n  ),\n  10\n)\n```\n\nBut `one_of` will support not just finite enumerables, but streams.\nSo below we calculate the first ten triples for all positive integers.\n\nIts worth noting that here, instead of using just using the `query` function,\nwe create an unresolved query, and then run it.\nThis allows us to reuse the query below.\n\n```elixir\nquery =\n  Query.new(\n    term(\n      And.new([\n        OneOf.new(a, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(b, Stream.iterate(1, &(&1 + 1))),\n        OneOf.new(c, Stream.iterate(1, &(&1 + 1))),\n        is(true, fn a, b -> a   Integer.pow(c, 2) - Integer.pow(a, 2) - Integer.pow(b, 2) end)\n      ])\n    )\n  )\n\nResult.run(query, 10)\n```\n\nNow that we have the unresolved query we can calculate even more, and pull the first 100.\nNote that this will take a few seconds.\nThe process is brute force, so as you request more results, more combinations of `a`,\n`b`, and `c` will have to be tested.\n\n```elixir\nResult.run(query, 100)\n```","ref":"pythagorean_triples.html#queries-that-calculate-values"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}